%% Function that solves the consumption path
function [path_out]=compute_hr_path(c_in,E_v_J,hrss,paths,parameters)
% computes a path of valuations given path of consumption, 
% uses prices to spit issuances and new path of consumption
% Jump in valuations at time zero
%
% Inputs:
% c_in  : uses consumption of RSS-path as initial guess
% J_v_in: uses Jump in value of RSS as initial guess
% hrss: uses Jump in value of RSS as initial guess
%
% Output
% path_out: structure with solution

%% Issues to fix...
% do we start from RSS? or we jump somewhere else? 
% multiple shocks...

%% Warmup
% Unpack Model Parameters
parameter_unpack ;
time_preallocate ;

% Unpack Parameters
rho         = parameters.rho        ;
sigma       = parameters.gamma      ;
phi         = parameters.phi        ; 
U_p_ratio   = parameters.U_p_ratio  ;
dU          = parameters.dU         ;
N_r         = parameters.N_r        ;
N_y         = parameters.N_y        ;
prob_mat    = parameters.prob_mat   ;

% Unpack Risky Steady State
psi_hrss   = hrss.psi_hrss; 
v_hrss     = hrss.v_hrss  ;

%% Step 1: Construct Guesses
% updates

% Check if double counting
r_n(:,1:N_t-1)      = rho+sigma*(c_in(:,2:N_t)./c_in(:,1:N_t-1)-1)/dt; 
r_n(1:N_r*N_y,N_t)  = rho;
r_hrss_adj          = r_n(1:N_r*N_y,N_t);

%% Step 2: For each shock, backout the paths
% Solve Values backward in time, given HRSS
ii=0;
v_hr_n=zeros(N_tau,N_t,N_y*N_r); % Issue with multiple shocks
for rr=1:N_r
    for yy=1:N_y
        ii=ii+1;
        r_adj                   = ones(N_tau,1)*r_n(ii,:)+phi*(1-E_v_J)*ones(1,N_t); % adjuste rate for HJB equation
        v_hr_n(:,:,ii)          = solve_gHJB_path(r_adj,r_hrss_adj(ii)+phi*(1-E_v_J),parameters)     ; 
    end
end
aux=[]; ii=0;
for rr=1:N_r
    for yy=1:N_y
        ii=ii+1;
        aux=[aux  v_hr_n(:,1,ii)]; 
    end 
end
v_0_mat   = aux;

%% Step 4: Solving for Path
f_0=hrss.f_0; % Use SS as initial condition...
% check
% check1=hrss.f_0-paths.rss.f_rss;
% check2=hrss.f_0-hrss_out.f_hrss;

% Updates values
c_out=zeros(N_r*N_y,N_t);
f_out=zeros(N_tau,N_t,N_r*N_y);
iota_out=zeros(N_tau,N_t,N_r*N_y);
ii=0;
for rr=1:N_r
    for yy=1:N_y
        ii=1+ii;
        path_out=solve_path(rr,yy,f_0,v_hr_n(:,:,ii),paths,parameters)  ;
            
        %% Step 5: Packaging Output
        c_out(ii,:)            = [path_out.c_n]      ;
        f_out(:,:,ii)          = [path_out.f_n]      ;
        iota_out(:,:,ii)       = [path_out.iota_n]   ;
        path_out.v_out(:,:,ii) = [v_hr_n(:,:,ii)]    ;
    end
end
path_out.c_out   = c_out                ;
path_out.f_out   = f_out                ;
path_out.iota_out= iota_out             ;
path_out.r_n     = r_n                  ;
path_out.v_n     = v_hr_n               ;
path_out.v_0_mat = v_0_mat              ;



% %% Step 3: Solving for Risky Steady State Value
% % [SB: fix here...]
% r_hrss                  = rho-phi*E_v_J                                    ; % phi effect inside..
% hrss_out=solve_hrss(psi_hrss*prob_mat,0*v_0_mat*prob_mat,r_hrss,parameters); % <- 0 because jump already in value...
% 
% % v_rss    = solve_HJB_rss(r_hrss_adj,v_0_mat*prob_mat,parameters)   ; % HJB: Solution at Steady State
% % iota_rss = 1/lambda_bar*(1-v_0_mat*prob_mat./psi_hrss*prob_mat); % Optimal policies
% % f_rss     = flip(cumsum(flip(iota_rss)))*dt      ; % KFE
% % c_rss = y_ss  - f_rss(1) + sum((psi_rss.*(1-1/2*lambda_bar.*iota_rss).*iota_rss)-delta*f_rss)*dt; % Consumption
% % r_rss = rho;
% % 
% % if c_rss<=0 % The case lambda <lambda_min
% %     warning('Negative Consumption at RSS');
% % end
% % hrss_out.c_rss=c_rss;
% % hrss_out.iota_rss=iota_rss;
% % hrss_out.v_rss=v_rss;
% % hrss_out.f_rss=f_rss;
% % hrss_out.r_rss=r_rss;
% 
% %% Step 5: Packaging Output
% ii=0;
% for rr=1:N_r
%     for yy=1:N_y
%         ii=1+ii;
%         path_out.v_out(:,:,ii) = [hrss.v_hrss  v_hr_n(:,:,ii)]    ;
%     end
% end
% path_out.c_out   = c_out                ;
% path_out.f_out   = f_out                ;
% path_out.iota_out= iota_out             ;
% path_out.r_n     = r_n                  ;
% path_out.v_n     = v_hr_n               ;