%% Compute the HRSS path, prior to consudering risk..
function [hr]=compute_psi_hrss_paths(hr,Phi,paths,parameters)
plotit=0;
% initial guess for the actual price...or, if there's no default risk, this
% object is used
% hr: some guess for initial prices...usually, rss, or previous loop
% Phi: probability of default... 

% unpack paramters
time_preallocate;
phi     = parameters.phi     ;
N_r     = parameters.N_r     ;
N_y     = parameters.N_y     ;
r_bar_ss= parameters.r_bar_ss;
prob_mat=parameters.prob_mat ;

%We take the shocks from the hrss, not the rss
exo_hr_path_construct  ; % output
paths.y_n=y_n          ; % output
paths.r_bar_n=r_bar_n  ; % output

% update r_bar_n
r_bar_n=paths.r_bar_n;

% initializing guess
psi_J_hrss=hr.psi_J_rss     ; % consider the jump, an initial guess
psi_hrss=hr.psi_rss         ; % provide a steady state for the prce

% initialize the variables
psi_hr_n=zeros(N_tau,N_t,N_r); % intializing prices, for each path of rates
psi_hr_0_mat=zeros(N_tau,N_r); % intializing prices, for each maturity

% In this first step, we compute the price system, assumming homothetic risk 

% Expected Jump, evaluated at each value             
% [SB] check if it works with a default prob......
E_psi_J=psi_J_hrss*sum(Phi.*prob_mat,2);
        
for iter = 1:parameters.hrss_iter % change parameter to 1 here for Galo's method.
        for rr=1:N_r                                                            
            r_bar_adj   = r_bar_n(rr,:)+ phi*(1-E_psi_J); % update path rate, to consider jump
            r_bar_adj_ss= r_bar_ss     + phi*(1-E_psi_J); % update steady-state and account for growth
            psi_hr_n(:,:,rr)  = solve_gHJB_path(r_bar_adj,r_bar_adj_ss,parameters) ; % construct prices
            psi_hr_0_mat(:,rr)= psi_hr_n(:,1,rr)                                   ; % save initial price      
        end
               
        % Update only in Saki's method
        if parameters.hrss_update == 1  % update only in Saki's method
            psi_rf_hrss_out= solve_HJB_rss(r_bar_ss,psi_hr_0_mat,parameters);               % after the path is computed, the risky-risky steady state is build
            residual=max(max(abs(psi_rf_hrss_out./psi_hrss-1)));
            psi_hrss=psi_rf_hrss_out;
            psi_J_hrss  = psi_hr_0_mat./psi_hrss                     ;
            if iter>10
                if residual<parameters.tol_path
                    % iter=max_iter+1;
                    disp('Converged in less than max iterations');
                    break
                else
                end
            end
        else
            break
        end
end

if plotit==1
    figure
    plot(psi_hr_0_mat(:,:)); hold on; plot(psi_hrss,'k--');
end

% Figure
if plotit==1
    if N_r>1
        psi_hr_0_mat(:,rr)
        surf(psi_n(:,:,1),'LineStyle','None'); hold on; surf(psi_hr_n(:,:,1),'LineStyle','None'); alpha(0.5)
        ylabel('Maturity'); xlabel('Period'); title('$\psi_t(\tau)$','interpreter','latex');    
    else
        figure
        subplot(1,2,1);
        surf(psi_n(:,:,1),'LineStyle','None'); hold on; surf(psi_hr_n(:,:,1),'LineStyle','None'); alpha(0.5)
        ylabel('Maturity'); xlabel('Period'); title('$\psi_t(\tau)$','interpreter','latex');    
        subplot(1,2,2);
        surf(psi_n(:,:,end),'LineStyle','None'); hold on; surf(psi_hr_n(:,:,end),'LineStyle','None'); alpha(0.5)
        ylabel('Maturity'); xlabel('Period'); title('$\psi_t(\tau)$','interpreter','latex');    
    end
end
% store results in structure that is used for prices
hr.psi_hr_0_mat= psi_hr_0_mat                                    ;  
hr.psi_hrss    = psi_hrss ;%solve_HJB_rss(r_bar_ss,psi_hr_0_mat,parameters) ;  
hr.psi_J_rss   = psi_hr_0_mat./psi_hrss                       ;       
hr.psi_hr_n    = psi_hr_n                                        ;